import typing
from google.cloud import logging
from google.cloud.logging.resource import Resource


class Logging:
    client = None
    logger = None

    _name: str = None
    _common_resource = None

    def __init__(self, name: str, namespace: str = 'default'):
        self.client = logging.Client()

        self._name = name

        self.logger = self.client.logger(self._name)

        self._common_resource = self._build_resource(name, namespace)

    @staticmethod
    def _build_resource(identifier, namespace: str = 'default', location: str = 'global'):
        return Resource(
            type='generic_node',
            labels={
                'location': location,
                'namespace': namespace,
                'node_id': identifier
            }
        )

    def log(self,
            data: typing.Union[str, typing.Dict],
            resource: typing.Union[Resource, typing.Dict] = None):

        resource_to_send = resource if resource is not None else self._common_resource

        if data is typing.Dict:
            self.logger.log_struct(data, resource=resource_to_send)
        else:
            self.logger.log_text(data, resource=resource_to_send)

        return True

    def log_request(self, uri, method, body, request_type: str = 'REQUEST'):
        self.logger.log_struct({
            'uri': uri,
            'method': method,
            'body': body,
            'type': request_type
        }, resource=self._common_resource,
        severity= 'ERROR' if body is not None and 'error' in body else None)

        return True
